<?php

require_once(ABSPATH . 'wp-includes/class-snoopy.php');

class KnsprDailyboothFetcher {
    private $username = null;

    private $cacheTime = 43200;
    private $cachePath = 'wp-content/plugins/dailybooth/cache/';

    private $rssUrl = 'http://dailybooth.com/rss/%s.rss';
    private $maxTrys = 5;

    public function __construct($username) {
        $this->username = $username;
    }

    public function getRecentPictures($count = 1) {
        if(!$this->availableFromCache()) {
            $pictures = $this->parseRssFeed();
            $this->updateCache($pictures);
        }

        return $this->getFromCache($count);
    }

    public function getLocalName(array $picture) {
        return $this->cachePath . md5($picture['link']) .'.jpg';
    }

    private function availableFromCache() {
        if(!file_exists($this->cachePath . 'cache')) {
            return false;
        }

        if(filemtime($this->cachePath . 'cache') + $this->cacheTime < time()) {
            return false;
        }

        return true;
    }

    private function getFromCache($count) {
        $data = file_get_contents($this->cachePath . 'cache');
        $array = unserialize($data);

        return array_slice($array, 0, $count);
    }

    private function updateCache(array $pictures) {
        foreach($pictures as $i => $pic) {
            $localName = $this->getLocalName($pic);

            if(!file_exists($localName) || filesize($localName) == 0) {
                $this->downloadFile($pic['image'], $localName);
            }
        }

        file_put_contents($this->cachePath . 'cache', serialize($pictures));
    }

    private function downloadFile($source, $destination, $try = 0) {
        if($try >= $this->maxTrys) {
            return;
        }

        $this->checkDirectory();

        $snoopy = new Snoopy();
        $snoopy->referer = $source;
        $snoopy->fetch($source);

        file_put_contents($destination, $snoopy->results);
    }

    private function checkDirectory() {
        if(!is_dir($this->cachePath)) {
            mkdir($this->cachePath, 0777, true) or print 'Could not create cache directory. Please create "'. $this->cachePath .'" and chmod it writable (0777).';
        }
    }

    private function parseRssFeed() {
        $url = sprintf($this->rssUrl, $this->username);
        $input = file_get_contents($url);

        preg_match_all("/<link>(.*)<\/link>.*img src=\"(.*)\"/isU", $input, $matches);
        $count = count($matches[0]);

        for($i = 0; $i < $count; $i++) {
            $output[] = array(
                'link' => $matches[1][$i],
                'image' => $matches[2][$i]
            );
        }

        return $output;
    }
}